#include <Servo.h>

// Define servo objects
Servo servoX;  // X rotate servo on pin D6
Servo servoZ;  // Z rotate servo on pin D5

// Define servo control variables
int posX = 90;  // Initial position for X servo
int posZ = 90;  // Initial position for Z servo

// Define angle limiters for Z servo
int minAngleZ = 120;  // Minimum angle for Z servo
int maxAngleZ = 170; // Maximum angle for Z servo

// Define angle limiters for X servo
int minAngleX = 35;  // Minimum angle for X servo
int maxAngleX = 85; // Maximum angle for X servo

void setup() {
  // Attach servos to their respective pins
  servoX.attach(D6);  // D6
  servoZ.attach(D5);  // D5
  
  // Initialize serial communication
  Serial.begin(9600);
  
  // Seed random number generator with analog pin 0 value
  randomSeed(analogRead(0));
}

void loop() {
  // Generate random target positions within defined limits for X and Z servos
  int targetX = random(minAngleX, maxAngleX);  // Random position within limits
  int targetZ = random(minAngleZ, maxAngleZ);  // Random position within limits
  
  // Generate random speeds for X and Z servos
  int speedX = random(1, 5);  // Random speed between 1 and 4
  int speedZ = random(1, 5);  // Random speed between 1 and 4
  
  // Move servos to random target positions smoothly
  moveServoSmoothly(servoX, posX, targetX, speedX);
  moveServoSmoothly(servoZ, posZ, targetZ, speedZ);
  
  // Update current positions
  posX = targetX;
  posZ = targetZ;
  
  // Pause before next movement
  int delayTime = (targetZ < (minAngleZ + (maxAngleZ - minAngleZ) / 3)) ? random(500, 1000) : random(1000, 3000);  // Less time if near the bottom
  delay(delayTime);  // Random delay between 1 to 3 seconds
}

// Function to smoothly move a servo from current position to target position at specified speed
void moveServoSmoothly(Servo &servo, int &currentPos, int targetPos, int speed) {
  int steps = abs(targetPos - currentPos);  // Calculate number of steps needed
  for (int i = 0; i <= steps; i++) {
    float progress = (float)i / steps;
    int newPos = currentPos + (targetPos - currentPos) * smoothStep(progress);
    servo.write(newPos);
    delay(speed * 10);  // Adjust delay for smoother effect
  }
  currentPos = targetPos;
}

// Smooth interpolation function
float smoothStep(float t) {
  return t * t * (3 - 2 * t);
}
